/*  This file is part of VIM Pal.
 *
 *  VIM Pal is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  VIM Pal is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with VIM Pal.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "fileutils.h"
#include "vimutils.h"
#include "settings.h"

#include <QProcess>
#include <QMessageBox>
#include <QMenu>
#include <QSignalMapper>


FileUtils* FileUtils::sInstance = 0;


bool FileUtils::init(QWidget *parent)
{
    if (sInstance)
    {
        return false;
    }

    sInstance = new FileUtils(parent);

    return true;
}

FileUtils *FileUtils::instance()
{
    return sInstance;
}

void FileUtils::addFileActions(const QString &fname, QMenu *menu)
{
    QList<CmdData> commands;
    CmdData firstCommand;

    foreach (const Settings::Tool& t, Settings::instance()->tools())
    {
        bool ok = false;

        if (!t.Filter.isEmpty())
        {
            foreach (const QString& i, t.Filter.split(";"))
            {
                if (QRegExp(i.trimmed(), Qt::CaseInsensitive, QRegExp::Wildcard).exactMatch(fname))
                {
                    ok = true;
                    break;
                }
            }
        }
        else
        {
            ok = true;
        }

        if (ok)
        {
            QString cmd = t.Command;

            cmd.replace("%s", fname);

            if (t.ByDefault && firstCommand.Command.isEmpty())
            {
                firstCommand.Name = t.Name;
                firstCommand.Command = cmd;
            }
            else
            {
                commands.append(CmdData(cmd, t.Name));
            }
        }
    }

    QAction* a;

    if (!firstCommand.Command.isEmpty())
    {
        a = menu->addAction(tr("Open with %1").arg(firstCommand.Name), mCmdMapper, SLOT(map()));

        mCmdMapper->setMapping(a, firstCommand.Command);
    }

    a = menu->addAction(tr("&Open in Vim"), mOpenMapper, SLOT(map()));

    mOpenMapper->setMapping(a, fname);

    if (!commands.isEmpty())
    {
        QMenu* m = menu->addMenu(tr("Open &with"));

        foreach (const CmdData& cmd, commands)
        {
            a = m->addAction(cmd.Name, mCmdMapper, SLOT(map()));

            mCmdMapper->setMapping(a, cmd.Command);
        }
    }
}

void FileUtils::openFile(const QString &fname)
{
    QString cmd;

    foreach (const Settings::Tool& t, Settings::instance()->tools())
    {
        if (t.ByDefault)
        {
            bool ok = false;

            if (!t.Filter.isEmpty())
            {
                foreach (const QString& i, t.Filter.split(";"))
                {
                    if (QRegExp(i.trimmed(), Qt::CaseInsensitive, QRegExp::Wildcard).exactMatch(fname))
                    {
                        ok = true;
                        break;
                    }
                }
            }
            else
            {
                ok = true;
            }

            if (ok)
            {
                cmd = t.Command;
                cmd.replace("%s", fname);

                break;
            }
        }
    }

    if (!cmd.isEmpty())
    {
        runCommand(cmd);
    }
    else
    {
        openFileInVim(fname);
    }
}

/* --------------------------------------------------------------------- */

void FileUtils::openFileInVim(const QString &fname)
{
    VimUtils::openFile(fname);
}

void FileUtils::runCommand(const QString &cmd)
{
    if (!QProcess::startDetached(cmd))
    {
        QMessageBox::critical(mParent, tr("Error"), tr("Failed to launch external tool."));
    }
}

FileUtils::FileUtils(QWidget *parent)
    :QObject(parent), mParent(parent)
{
    mOpenMapper = new QSignalMapper(this);
    mCmdMapper = new QSignalMapper(this);

    connect(mOpenMapper, SIGNAL(mapped(QString)), this, SLOT(openFileInVim(QString)));
    connect(mCmdMapper, SIGNAL(mapped(QString)), this, SLOT(runCommand(QString)));
}
