/*  This file is part of VIM Pal.
 *
 *  VIM Pal is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  VIM Pal is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with VIM Pal.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "profile.h"
#include "settings.h"

#include <QFileInfo>
#include <QStandardItemModel>
#include <QSettings>
#include <QFileIconProvider>


Profile::Profile(const Data& data)
    :mData(data)
{
    mQuickFileGroups.append(QObject::tr("Default"));
    mQuickFiles.append(new QStandardItemModel);
}

Profile::~Profile()
{
    qDeleteAll(mQuickFiles);
}

void Profile::setName(const QString &name)
{
    mData.Title = name;
}

void Profile::setServer(const QString &name)
{
    mData.Server = name;
}

void Profile::setDir(const QString &dir)
{
    mData.Dir = dir;

    saveSettingsItem("root_dir", dir);
}

void Profile::setIndex(int idx)
{
    mData.Index = idx;
}

void Profile::setExpandedDirs(const QStringList &exp)
{
    mData.ExpandedDirs = exp;

    saveSettingsItem("expanded_dirs", exp);
}

QStandardItemModel *Profile::quickFileModel(int index) const
{
    return mQuickFiles.at(index);
}

void Profile::addQuickFileGroup(const QString &name)
{
    mQuickFileGroups.append(name);
    mQuickFiles.append(new QStandardItemModel);
    mData.ExpandedQuickDirs.append(QStringList());
}

void Profile::removeQuickFileGroup(int index)
{
    mQuickFileGroups.removeAt(index);
    delete mQuickFiles.takeAt(index);
    mData.ExpandedQuickDirs.removeAt(index);
}

void Profile::renameQuickFileGroup(int index, const QString &name)
{
    mQuickFileGroups[index] = name;
}

void Profile::loadQuickFiles(const QVariant& set)
{
    qDeleteAll(mQuickFiles);

    mQuickFileGroups.clear();
    mQuickFiles.clear();

    switch (set.type())
    {
        // Old settings
        case QVariant::StringList:
        {
            QStandardItemModel* items = new QStandardItemModel;

            foreach (const QString& i, set.toStringList())
            {
                items->appendRow(fileItem(i));
            }

            mQuickFileGroups.append(QObject::tr("Default"));
            mQuickFiles.append(items);

            break;
        }

        case QVariant::ByteArray:
        {
            QDataStream str(set.toByteArray());

            qint32 c;

            str >> c;

            for (int i=0; i<c; i++)
            {
                QStandardItemModel* model = new QStandardItemModel;
                QString name;

                str >> name;
                makeTree(str, model);

                mQuickFileGroups.append(name);
                mQuickFiles.append(model);
            }
            break;
        }

        // Invalid settings
        default:
            mQuickFileGroups.append(QObject::tr("Default"));
            mQuickFiles.append(new QStandardItemModel);
            break;
    }

    while (mData.ExpandedQuickDirs.length() < mQuickFileGroups.length())
    {
        mData.ExpandedQuickDirs.append(QStringList());
    }
}

QByteArray Profile::getQuickFiles()
{
    QByteArray data;
    QDataStream str(&data, QIODevice::WriteOnly);

    const qint32 c = mQuickFileGroups.length();

    str << c;

    for (int i=0; i<c; i++)
    {
        str << mQuickFileGroups.at(i);

        collectTreeItems(str, mQuickFiles.at(i));
    }

    return data;
}

void Profile::saveProfile(QSettings &set)
{
    set.setValue("name", mData.Title);
    set.setValue("server_name", mData.Server);
    set.setValue("root_dir", mData.Dir);
    set.setValue("expanded_dirs", mData.ExpandedDirs);
    set.setValue("expanded_quick_dirs", mData.ExpandedQuickDirs);
    set.setValue("current_quick_files", mData.QuickFileGroup);

    set.setValue("quick_files", getQuickFiles());
}

void Profile::saveQuickFiles()
{
    saveSettingsItem("quick_files", getQuickFiles());
}

void Profile::setExpandedQuickDirs(int index, const QStringList &items)
{
    mData.ExpandedQuickDirs[index] = items;

    saveSettingsItem("expanded_quick_dirs", mData.ExpandedQuickDirs);
}

void Profile::setQuickFileGroup(int index)
{
    mData.QuickFileGroup = index;

    saveSettingsItem("current_quick_files", mData.QuickFileGroup);
}

/* --------------------------------------------------------------------- */

void Profile::makeTree(QDataStream& str, QStandardItemModel *model, QStandardItem* parent)
{
    qint32 c;
    QFileIconProvider icons;

    str >> c;

    for (int i=0; i<c; i++)
    {
        QString file;
        qint8 type;
        QStandardItem* it;

        str >> type;
        str >> file;

        switch (type)
        {
            case 1:
                it = new QStandardItem;
                it->setData(file, Qt::DisplayRole);
                it->setData(icons.icon(QFileIconProvider::Folder), Qt::DecorationRole);
                it->setFlags(Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsEnabled | Qt::ItemIsEditable | Qt::ItemIsDropEnabled);
                break;

            case 2:
                it = fileItem(file);
                break;

            default:
                qWarning("Invalid settings format! Cannot read quick file tree!");
                return;
        }

        if (parent)
        {
            parent->appendRow(it);
        }
        else
        {
            model->appendRow(it);
        }

        makeTree(str, model, it);
    }
}

void Profile::collectTreeItems(QDataStream& str, QStandardItemModel *model, const QModelIndex &parent)
{
    const qint32 c = model->rowCount(parent);

    str << c;

    for (int i=0; i<c; i++)
    {
        QModelIndex idx = model->index(i, 0, parent);
        qint8 type = idx.data(Qt::UserRole).toString().isEmpty() ? 1 : 2;

        str << type;

        str << ((type == 1) ?
                   idx.data(Qt::DisplayRole).toString() :
                   idx.data(Qt::UserRole).toString());

        collectTreeItems(str, model, idx);
    }
}

QStandardItem *Profile::fileItem(const QString &fname)
{
    QFileIconProvider icons;

    QStandardItem* it = new QStandardItem;

#if QT_VERSION >= 0x050000
    it->setFlags(Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsEnabled | Qt::ItemNeverHasChildren);
#else
    it->setFlags(Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsEnabled);
#endif

    it->setData(fname, Qt::UserRole);
    it->setData(fname, Qt::ToolTipRole);
    it->setData(QFileInfo(fname).fileName(), Qt::DisplayRole);
    it->setData(icons.icon(QFileInfo(fname)), Qt::DecorationRole);

    return it;
}

void Profile::saveSettingsItem(const QString &name, const QVariant &value)
{
    QSettings set;

    int count = set.beginReadArray("profiles");

    if (count < mData.Index + 1)
    {
        count = mData.Index + 1;
    }

    set.endArray();

    set.beginWriteArray("profiles", count);
    set.setArrayIndex(mData.Index);
    set.setValue(name, value);
    set.endArray();
}
