/*  This file is part of VIM Pal.
 *
 *  VIM Pal is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  VIM Pal is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with VIM Pal.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "profiledialog.h"
#include "profile.h"

#include <QStandardItemModel>
#include <QPushButton>


ProfileDialog::ProfileDialog(QWidget* parent)
    :QDialog(parent)
{
    setupUi(this);

    buttonBox->button(QDialogButtonBox::Ok)->setText(tr("&OK"));
    buttonBox->button(QDialogButtonBox::Cancel)->setText(tr("&Cancel"));

    mProfModel = new QStandardItemModel(this);

    profileList->setModel(mProfModel);

    connect(profileList->selectionModel(), SIGNAL(currentChanged(QModelIndex, QModelIndex)),
            this, SLOT(currentChanged(QModelIndex, QModelIndex)));
}

void ProfileDialog::addProfile(const QString &name, const QString &server, int index)
{
    QList<QStandardItem*> item;

    item.append(new QStandardItem(name));
    item.append(new QStandardItem(server));
    item.append(new QStandardItem(QString::number(index)));

    mProfModel->appendRow(item);
}

QList<ProfileDialog::Profile> ProfileDialog::profiles() const
{
    QList<Profile> prof;
    const int c = mProfModel->rowCount();

    for (int i=0; i<c; i++)
    {
        Profile p;

        p.Name = mProfModel->index(i, 0).data().toString();
        p.Server = mProfModel->index(i, 1).data().toString();
        p.Index = mProfModel->index(i, 2).data().toInt();

        prof.append(p);
    }

    return prof;
}

void ProfileDialog::showEvent(QShowEvent *e)
{
    profileList->header()->hideSection(2);

    mProfModel->setHeaderData(0, Qt::Horizontal, tr("Profile name"), Qt::DisplayRole);
    mProfModel->setHeaderData(1, Qt::Horizontal, tr("Vim name"), Qt::DisplayRole);

    QDialog::showEvent(e);
}

/* --------------------------------------------------------------------- */

void ProfileDialog::on_addProfile_clicked()
{
    QList<QStandardItem*> item;

    item.append(new QStandardItem(tr("New profile")));
    item.append(new QStandardItem("GVIM"));
    item.append(new QStandardItem("-1"));

    mProfModel->appendRow(item);

    profileList->selectionModel()->setCurrentIndex(mProfModel->index(mProfModel->rowCount() - 1, 0),
        QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Current | QItemSelectionModel::Rows);

    profileList->edit(mProfModel->index(mProfModel->rowCount() - 1, 0));
}

void ProfileDialog::on_removeProfile_clicked()
{
    if (profileList->currentIndex().isValid() && mProfModel->rowCount() > 1)
    {
        mProfModel->removeRow(profileList->currentIndex().row());

        currentChanged(profileList->currentIndex());
    }
}

void ProfileDialog::on_moveUp_clicked()
{
    if (profileList->currentIndex().isValid())
    {
        const int row = profileList->currentIndex().row();

        if (row > 0)
        {
            mProfModel->insertRow(row - 1, mProfModel->takeRow(row));

            profileList->selectionModel()->setCurrentIndex(mProfModel->index(row - 1, 0),
                QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Current | QItemSelectionModel::Rows);
        }
    }
}

void ProfileDialog::on_moveDown_clicked()
{
    if (profileList->currentIndex().isValid())
    {
        const int row = profileList->currentIndex().row();

        if (row < mProfModel->rowCount() - 1)
        {
            mProfModel->insertRow(row + 1, mProfModel->takeRow(row));

            profileList->selectionModel()->setCurrentIndex(mProfModel->index(row + 1, 0),
                QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Current | QItemSelectionModel::Rows);
        }
    }
}

void ProfileDialog::currentChanged(const QModelIndex &index, const QModelIndex &previous)
{
    Q_UNUSED(previous);

    if (index.isValid())
    {
        removeProfile->setEnabled(mProfModel->rowCount() > 1);
        moveUp->setEnabled(index.row() > 0);
        moveDown->setEnabled(index.row() < mProfModel->rowCount() - 1);
    }
    else
    {
        removeProfile->setEnabled(false);
        moveUp->setEnabled(false);
        moveDown->setEnabled(false);
    }
}
