/*  This file is part of VIM Pal.
 *
 *  VIM Pal is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  VIM Pal is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with VIM Pal.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "profilemodel.h"
#include "profiledialog.h"
#include "profile.h"

#include <QSettings>
#include <QDir>


ProfileModel::ProfileModel(QObject *parent)
    :QAbstractListModel(parent)
{
    QSettings set;

    const int c = set.beginReadArray("profiles");

    if (!c)
    {
        Profile::Data d;

        d.Index = 0;
        d.Title = tr("Default");
        d.Server = "GVIM";
        d.Dir = QDir::homePath();
        d.QuickFileGroup = 0;
        d.ExpandedQuickDirs.append(QStringList());

        mProfiles.append(new Profile(d));
    }
    else
    {
        for (int i=0; i<c; i++)
        {
            set.setArrayIndex(i);

            Profile::Data d;

            d.Index = i;
            d.Title = set.value("name", tr("Default")).toString();
            d.Server = set.value("server_name", "GVIM").toString();
            d.Dir = set.value("root_dir", QDir::homePath()).toString();
            d.ExpandedDirs = set.value("expanded_dirs").toStringList();
            d.QuickFileGroup = set.value("current_quick_files", 0).toInt();
            d.ExpandedQuickDirs = set.value("expanded_quick_dirs", QVariantList() << QStringList()).toList();

            Profile* p = new Profile(d);

            p->loadQuickFiles(set.value("quick_files"));

            mProfiles.append(p);
        }
    }

    set.endArray();
}

ProfileModel::~ProfileModel()
{
    qDeleteAll(mProfiles);
}

int ProfileModel::rowCount(const QModelIndex &parent) const
{
    return parent.isValid() ? 0 : mProfiles.length();
}

QVariant ProfileModel::data(const QModelIndex &index, int role) const
{
    if (role == Qt::DisplayRole)
    {
        return mProfiles.at(index.row())->data().Title;
    }
    else if (role == Qt::DecorationRole)
    {
        return QIcon::fromTheme("dialog-information", QIcon(":/img/dialog-information.png"));
    }
    else
    {
        return QVariant();
    }
}

Qt::ItemFlags ProfileModel::flags(const QModelIndex &index) const
{
    if (index.isValid())
    {
        return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
    }
    else
    {
        return Qt::NoItemFlags;
    }
}

bool ProfileModel::profilesDialog(QWidget *parent, int& saveIndex)
{
    ProfileDialog dlg(parent);

    const int c = mProfiles.length();

    for (int i=0; i<c; i++)
    {
        dlg.addProfile(mProfiles.at(i)->data().Title, mProfiles.at(i)->data().Server, i);
    }

    if (dlg.exec() == QDialog::Accepted)
    {
        QList<Profile*> newProfs;

        bool indexFound = false;

        foreach (const ProfileDialog::Profile p, dlg.profiles())
        {
            if (p.Index == -1)
            {
                Profile::Data np;

                np.Index = newProfs.length();
                np.Title = p.Name;
                np.Server = p.Server;
                np.QuickFileGroup = 0;
                np.ExpandedQuickDirs.append(QStringList());

                newProfs.append(new Profile(np));
            }
            else
            {
                Profile* op = mProfiles.at(p.Index);

                op->setIndex(newProfs.length());
                op->setName(p.Name);
                op->setServer(p.Server);

                if (!indexFound && p.Index == saveIndex)
                {
                    saveIndex = op->data().Index;

                    indexFound = true;
                }

                newProfs.append(op);
            }
        }

        if (!indexFound)
        {
            // Current profile deleted. Setting first one as the new current.

            saveIndex = 0;
        }

        QSet<Profile*> oldProfs = mProfiles.toSet();

        oldProfs.subtract(newProfs.toSet());

        qDeleteAll(oldProfs);

        beginResetModel();

        mProfiles = newProfs;

        endResetModel();

        {
            QSettings set;

            set.beginWriteArray("profiles", mProfiles.count());

            int idx = 0;

            foreach (Profile* p, mProfiles)
            {
                set.setArrayIndex(idx++);

                p->saveProfile(set);
            }

            set.endArray();
        }

        return true;
    }

    return false;
}

Profile *ProfileModel::profile(int index) const
{
    if (index >= 0 && index < mProfiles.length())
    {
        return mProfiles.at(index);
    }
    else
    {
        return 0;
    }
}
