/*  This file is part of VIM Pal.
 *
 *  VIM Pal is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  VIM Pal is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with VIM Pal.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "quickfileview.h"
#include "settings.h"
#include "profile.h"
#include "vimutils.h"
#include "fileutils.h"

#include <QTreeView>
#include <QVBoxLayout>
#include <QStandardItemModel>
#include <QFileInfo>
#include <QMenu>
#include <QInputDialog>
#include <QFileIconProvider>
#include <QTimer>


QuickFileView::QuickFileView(QWidget *parent)
    :QWidget(parent),
      mCurrentTab(-1), mCurrentModel(0), mCurrentProfile(0)
{
    mChangeTimer = new QTimer(this);

    mChangeTimer->setSingleShot(true);
    mChangeTimer->setInterval(1000);

    connect(mChangeTimer, SIGNAL(timeout()), this, SLOT(saveQuickFiles()));

    mQuickFileView = new QTreeView(this);

    mQuickFileView->setDragDropMode(QAbstractItemView::InternalMove);
    mQuickFileView->setSelectionMode(QAbstractItemView::ExtendedSelection);
    mQuickFileView->setEditTriggers(QAbstractItemView::DoubleClicked | QAbstractItemView::EditKeyPressed);
    mQuickFileView->setContextMenuPolicy(Qt::CustomContextMenu);
    mQuickFileView->setHeaderHidden(true);

    connect(mQuickFileView, SIGNAL(clicked(QModelIndex)), this, SLOT(quickFileClicked(QModelIndex)));
    connect(mQuickFileView, SIGNAL(activated(QModelIndex)), this, SLOT(quickFileActivated(QModelIndex)));
    connect(mQuickFileView, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(quickFileActivated(QModelIndex)));
    connect(mQuickFileView, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(contextMenu(QPoint)));

    mTabBar = new QTabBar(this);

    connect(mTabBar, SIGNAL(currentChanged(int)), this, SLOT(groupSelected(int)));

    QVBoxLayout* layout = new QVBoxLayout(this);

    layout->setContentsMargins(0, 0, 0, 0);
    layout->setSpacing(0);
    layout->addWidget(mTabBar);
    layout->addWidget(mQuickFileView);

    setFocusProxy(mQuickFileView);
}

void QuickFileView::setProfile(Profile *p)
{
    if (p->quickFileGroups().isEmpty())
    {
        qWarning("Invalid profile configuration. Panic and flee.");

        return;
    }
    else
    {
        if (mCurrentProfile && mCurrentTab != -1)
        {
            mCurrentProfile->setExpandedQuickDirs(mCurrentTab, getExpandedDirs());
            mCurrentProfile->setQuickFileGroup(mCurrentTab);
        }

        mCurrentProfile = p;
        mCurrentTab = mCurrentProfile->data().QuickFileGroup;

        // Just to be sure...
        if (mCurrentTab < 0 ||
                mCurrentTab >= mCurrentProfile->data().ExpandedQuickDirs.length() ||
                mCurrentTab >= mCurrentProfile->quickFileGroups().length())
        {
            mCurrentTab = 0;

            if (mCurrentProfile->data().ExpandedQuickDirs.length() < 1)
            {
                mCurrentProfile->setExpandedQuickDirs(0, QStringList());
            }
        }

        setCurrentModel(p->quickFileModel(mCurrentTab));

        mQuickFileView->setModel(mCurrentModel);

        expandDirs(mCurrentProfile->data().ExpandedQuickDirs.at(mCurrentTab).toStringList());

        mTabBar->blockSignals(true);

        while (mTabBar->count())
        {
            mTabBar->removeTab(0);
        }

        foreach (const QString& name, p->quickFileGroups())
        {
            mTabBar->addTab(name);
        }

        mTabBar->setVisible(mTabBar->count() > 1);
        mTabBar->setCurrentIndex(mCurrentTab);

        mTabBar->blockSignals(false);
    }
}

void QuickFileView::addFile(const QString &fname)
{
    if (findFile(fname))
    {
        return;
    }

    QStandardItem* it = new QStandardItem;
    QFileIconProvider icons;

#if QT_VERSION >= 0x050000
    it->setFlags(Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsEnabled | Qt::ItemNeverHasChildren);
#else
    it->setFlags(Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsEnabled);
#endif

    it->setData(fname, Qt::UserRole);
    it->setData(fname, Qt::ToolTipRole);
    it->setData(QFileInfo(fname).fileName(), Qt::DisplayRole);
    it->setData(icons.icon(QFileInfo(fname)), Qt::DecorationRole);

    addItem(it);
}

void QuickFileView::saveExpandedDirs()
{
    if (mCurrentProfile && mCurrentTab != -1)
    {
        mCurrentProfile->setExpandedQuickDirs(mCurrentTab, getExpandedDirs());
    }
}

void QuickFileView::saveQuickFiles()
{
    if (mCurrentProfile)
    {
        mCurrentProfile->saveQuickFiles();
    }
}

/* --------------------------------------------------------------------- */

void QuickFileView::quickFileClicked(const QModelIndex &idx)
{
    if (Settings::instance()->singleClickOpen())
    {
        quickFileActivated(idx);
    }
}

void QuickFileView::quickFileActivated(const QModelIndex &idx)
{
    const QString file = idx.data(Qt::UserRole).toString();

    if (!file.isEmpty())
    {
        FileUtils::instance()->openFile(file);
    }
}

void QuickFileView::contextMenu(const QPoint &pos)
{
    QModelIndex idx = mQuickFileView->indexAt(pos);
    QMenu m(this);

    if (idx.isValid())
    {
        mQuickFileView->selectionModel()->select(idx, QItemSelectionModel::ClearAndSelect);

        const QString file = idx.data(Qt::UserRole).toString();

        if (!file.isEmpty())
        {
            // A file

            FileUtils::instance()->addFileActions(file, &m);
        }
        else
        {
            // A folder

            m.addAction(tr("Rename"), this, SLOT(editCurrent()));
            m.addAction(tr("Add subfolder"), this, SLOT(addFolder()));
            m.addAction(tr("Sort subitems"), this, SLOT(sortItems()));
        }

        m.addSeparator();
        m.addAction(tr("Remove"), this, SLOT(removeCurrent()));
    }
    else
    {
        // Nothing

        mQuickFileView->selectionModel()->clear();

        m.addAction(tr("Add folder"), this, SLOT(addFolder()));
        m.addAction(tr("Sort items"), this, SLOT(sortItems()));
        m.addSeparator();
    }

    m.addAction(tr("Remove all"), this, SLOT(removeAll()));

    m.addSeparator();

    QMenu tm(tr("&Tabs"), this);

    m.addMenu(&tm);

    tm.addAction(tr("Add tab"), this, SLOT(addTab()));

    if (mTabBar->count() > 1)
    {
        tm.addAction(tr("Rename tab"), this, SLOT(renameTab()));
        tm.addAction(tr("Remove current tab"), this, SLOT(removeTab()));
    }

    m.exec(mQuickFileView->mapToGlobal(pos) + QPoint(1, 1));
}

void QuickFileView::addFolder()
{
    QInputDialog dlg(this);

    dlg.setOkButtonText(tr("&OK"));
    dlg.setCancelButtonText(tr("&Cancel"));
    dlg.setWindowTitle(tr("Create new folder"));
    dlg.setLabelText(tr("Enter the name for new folder:"));

    if (dlg.exec() == QDialog::Accepted)
    {
        QStandardItem* it = new QStandardItem;
        QFileIconProvider icons;

        it->setData(dlg.textValue(), Qt::DisplayRole);
        it->setData(icons.icon(QFileIconProvider::Folder), Qt::DecorationRole);
        it->setFlags(Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsEnabled | Qt::ItemIsEditable | Qt::ItemIsDropEnabled);

        addItem(it);
    }
}

void QuickFileView::removeAll()
{
    mCurrentModel->clear();
}

void QuickFileView::removeCurrent()
{
    bool found;

    do
    {
        const QModelIndexList list = mQuickFileView->selectionModel()->selectedIndexes();

        found = false;

        foreach (const QModelIndex& idx, list)
        {
            if (idx.isValid())
            {
                found = true;

                if (idx.parent().isValid())
                {
                    QStandardItem* it = mCurrentModel->itemFromIndex(idx.parent());

                    if (it)
                    {
                        qDeleteAll(it->takeRow(idx.row()));
                    }
                }
                else
                {
                    qDeleteAll(mCurrentModel->takeRow(idx.row()));
                }
            }
        }
    }
    while (found);
}

void QuickFileView::editCurrent()
{
    if (mQuickFileView->selectionModel()->hasSelection())
    {
        mQuickFileView->edit(mQuickFileView->selectionModel()->selectedIndexes().at(0));
    }
}

void QuickFileView::resetProfile()
{
    mCurrentProfile = 0;
    mCurrentModel = 0;
}

void QuickFileView::sortItems()
{
    if (mQuickFileView->selectionModel()->hasSelection())
    {
        mCurrentModel->itemFromIndex(mQuickFileView->selectionModel()->selectedIndexes().at(0))->sortChildren(0);
    }
    else
    {
        mCurrentModel->sort(0);
    }

    saveQuickFiles();
}

void QuickFileView::addTab()
{
    QInputDialog dlg(this);

    dlg.setOkButtonText(tr("&OK"));
    dlg.setCancelButtonText(tr("&Cancel"));
    dlg.setWindowTitle(tr("Add tab"));
    dlg.setLabelText(tr("Enter name for new tab:"));

    if (dlg.exec() == QDialog::Accepted)
    {
        mCurrentProfile->addQuickFileGroup(dlg.textValue());
        mCurrentProfile->saveQuickFiles();

        mTabBar->addTab(dlg.textValue());
        mTabBar->setCurrentIndex(mTabBar->count() - 1);

        mTabBar->setVisible(mTabBar->count() > 1);
    }
}

void QuickFileView::removeTab()
{
    const int idx = mTabBar->currentIndex();

    mCurrentProfile->removeQuickFileGroup(idx);
    mCurrentProfile->saveQuickFiles();

    mTabBar->blockSignals(true);
    mTabBar->removeTab(idx);
    mTabBar->blockSignals(false);
    mTabBar->setVisible(mTabBar->count() > 1);

    mCurrentTab = mTabBar->currentIndex();
    mCurrentModel = 0;

    setCurrentModel(mCurrentProfile->quickFileModel(mCurrentTab));

    mCurrentProfile->setQuickFileGroup(mCurrentTab);

    mQuickFileView->setModel(mCurrentModel);

    expandDirs(mCurrentProfile->data().ExpandedQuickDirs.at(mCurrentTab).toStringList());
}

void QuickFileView::renameTab()
{
    QInputDialog dlg(this);
    const int idx = mTabBar->currentIndex();

    dlg.setOkButtonText(tr("&OK"));
    dlg.setCancelButtonText(tr("&Cancel"));
    dlg.setWindowTitle(tr("Rename tab"));
    dlg.setLabelText(tr("Enter the new name for tab:"));
    dlg.setTextValue(mTabBar->tabText(idx));

    if (dlg.exec() == QDialog::Accepted)
    {
        mTabBar->setTabText(idx, dlg.textValue());

        mCurrentProfile->renameQuickFileGroup(idx, dlg.textValue());
        mCurrentProfile->saveQuickFiles();
    }
}

void QuickFileView::listChanged()
{
    if (!mChangeTimer->isActive())
    {
        mChangeTimer->start();
    }
}

void QuickFileView::groupSelected(int idx)
{
    if (mCurrentTab != -1)
    {
        mCurrentProfile->setExpandedQuickDirs(mCurrentTab, getExpandedDirs());
    }

    setCurrentModel(mCurrentProfile->quickFileModel(idx));

    mQuickFileView->setModel(mCurrentModel);

    mCurrentTab = idx;
    mCurrentProfile->setQuickFileGroup(idx);

    expandDirs(mCurrentProfile->data().ExpandedQuickDirs.at(mCurrentTab).toStringList());
}

void QuickFileView::addItem(QStandardItem *item)
{
    QModelIndex idx;

    if (mQuickFileView->selectionModel()->hasSelection())
    {
        idx = mQuickFileView->selectionModel()->selectedIndexes().at(0);

        if (!idx.data(Qt::UserRole).toString().isEmpty())
        {
            idx = idx.parent();
        }
    }

    if (idx.isValid())
    {
        mCurrentModel->itemFromIndex(idx)->appendRow(item);
    }
    else
    {
        mCurrentModel->appendRow(item);
    }
}

QStandardItem *QuickFileView::findFile(const QString &name, QStandardItem* parent) const
{
    if (parent)
    {
        const int c = parent->rowCount();

        for (int i=0; i<c; i++)
        {
            QStandardItem* it = parent->child(i, 0);

            if (it->data(Qt::UserRole).toString() == name)
            {
                return it;
            }

            if (it->hasChildren())
            {
                QStandardItem* ii = findFile(name, it);

                if (ii)
                {
                    return ii;
                }
            }
        }
    }
    else
    {
        const int c = mCurrentModel->rowCount();

        for (int i=0; i<c; i++)
        {
            QStandardItem* it = mCurrentModel->item(i, 0);

            if (it->data(Qt::UserRole).toString() == name)
            {
                return it;
            }

            if (it->hasChildren())
            {
                QStandardItem* ii = findFile(name, it);

                if (ii)
                {
                    return ii;
                }
            }
        }
    }

    return 0;
}

QStringList QuickFileView::getExpandedDirs(const QModelIndex& parent) const
{
    QStringList list;

    const int c = mCurrentModel->rowCount(parent);

    for (int i=0; i<c; i++)
    {
        const QModelIndex idx = mCurrentModel->index(i, 0, parent);

        if (idx.data(Qt::UserRole).toString().isEmpty())
        {
            if (mQuickFileView->isExpanded(idx))
            {
                list.append(idx.data(Qt::DisplayRole).toString());
            }

            list.append(getExpandedDirs(idx));
        }
    }

    return list;
}

void QuickFileView::expandDirs(const QStringList &dirs, const QModelIndex& parent)
{
    if (!parent.isValid())
    {
        mQuickFileView->collapseAll();
    }

    if (dirs.isEmpty())
    {
        return;
    }

    const int c = mCurrentModel->rowCount(parent);

    for (int i=0; i<c; i++)
    {
        const QModelIndex idx = mCurrentModel->index(i, 0, parent);

        if (idx.data(Qt::UserRole).toString().isEmpty() && dirs.contains(idx.data(Qt::DisplayRole).toString()))
        {
            mQuickFileView->expand(idx);

            expandDirs(dirs, idx);
        }
    }
}

void QuickFileView::setCurrentModel(QStandardItemModel *model)
{
    if (mCurrentModel)
    {
        disconnect(mCurrentModel, SIGNAL(rowsInserted(QModelIndex, int, int)), this, SLOT(listChanged()));
        disconnect(mCurrentModel, SIGNAL(rowsRemoved(QModelIndex, int, int)), this, SLOT(listChanged()));
        disconnect(mCurrentModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SLOT(listChanged()));
        disconnect(mCurrentModel, SIGNAL(modelReset()), this, SLOT(listChanged()));
    }

    mCurrentModel = model;

    connect(mCurrentModel, SIGNAL(rowsInserted(QModelIndex, int, int)), this, SLOT(listChanged()));
    connect(mCurrentModel, SIGNAL(rowsRemoved(QModelIndex, int, int)), this, SLOT(listChanged()));
    connect(mCurrentModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SLOT(listChanged()));
    connect(mCurrentModel, SIGNAL(modelReset()), this, SLOT(listChanged()));
}
