/*  This file is part of VIM Pal.
 *
 *  VIM Pal is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  VIM Pal is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with VIM Pal.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "settings.h"
#include "settingsdialog.h"

#include <QSettings>
#include <QDir>


Settings* Settings::sSettings = 0;


Settings *Settings::instance()
{
    if (!sSettings)
    {
        sSettings = new Settings;
    }

    return sSettings;
}

bool Settings::settingsDialog(QWidget *parent, QVariantMap& extraSettings)
{
    SettingsDialog dlg(parent);

    dlg.singleClickOpen->setChecked(mSingleClickOpen);
    dlg.changeDir->setChecked(mChangeDir);
    dlg.vimCommand->setText(mVimCommand);
    dlg.newTabs->setChecked(mOpenNewTabs);
    dlg.darkBackground->setChecked(mDarkBackground);
    dlg.showHidden->setChecked(mShowHidden);
    dlg.autoQuickList->setChecked(mAutoQuickList);
    dlg.asToolWindow->setChecked(mAsToolWindow);
    dlg.useSessions->setChecked(mUseSessions);
    dlg.oneVimWindow->setChecked(mOneVimWindow);
    dlg.sessionDir->setText(mSessionFileDir);
    dlg.profilesList->setChecked(!mProfilesIcons);
    dlg.profilesIcons->setChecked(mProfilesIcons);

    dlg.navBar->setChecked(extraSettings["navBar"].toBool());
    dlg.profileBar->setChecked(extraSettings["profileBar"].toBool());
    dlg.profileDock->setChecked(extraSettings["profileDock"].toBool());
    dlg.quickDock->setChecked(extraSettings["quickDock"].toBool());

    dlg.setTools(mTools);
    dlg.setLanguage(mLanguage);

    if (dlg.exec() == QDialog::Accepted)
    {
        mSingleClickOpen = dlg.singleClickOpen->isChecked();
        mChangeDir = dlg.changeDir->isChecked();
        mVimCommand = dlg.vimCommand->text();
        mOpenNewTabs = dlg.newTabs->isChecked();
        mDarkBackground = dlg.darkBackground->isChecked();
        mShowHidden = dlg.showHidden->isChecked();
        mAutoQuickList = dlg.autoQuickList->isChecked();
        mAsToolWindow = dlg.asToolWindow->isChecked();
        mUseSessions = dlg.useSessions->isChecked();
        mOneVimWindow = dlg.oneVimWindow->isChecked();
        mSessionFileDir = dlg.sessionDir->text();
        mProfilesIcons = dlg.profilesIcons->isChecked();

        extraSettings["navBar"] = dlg.navBar->isChecked();
        extraSettings["profileBar"] = dlg.profileBar->isChecked();
        extraSettings["profileDock"] = dlg.profileDock->isChecked();
        extraSettings["quickDock"] = dlg.quickDock->isChecked();

        mTools = dlg.getTools();
        mLanguage = dlg.getLanguage();

        QSettings set;

        set.setValue("single_click", mSingleClickOpen);
        set.setValue("change_dir", mChangeDir);
        set.setValue("vim_command", mVimCommand);
        set.setValue("new_tabs", mOpenNewTabs);
        set.setValue("dark_background", mDarkBackground);
        set.setValue("show_hidden", mShowHidden);
        set.setValue("auto_quicklist", mAutoQuickList);
        set.setValue("as_tool_window", mAsToolWindow);
        set.setValue("language", mLanguage);
        set.setValue("use_sessions", mUseSessions);
        set.setValue("one_vim_window", mOneVimWindow);
        set.setValue("session_file_dir", mSessionFileDir);
        set.setValue("profile_icon_view", mProfilesIcons);

        set.beginWriteArray("tools", mTools.count());

        for (int i=0, c=mTools.count(); i<c; i++)
        {
            set.setArrayIndex(i);

            set.setValue("name", mTools.at(i).Name);
            set.setValue("command", mTools.at(i).Command);
            set.setValue("filter", mTools.at(i).Filter);
            set.setValue("by_default", mTools.at(i).ByDefault);
        }

        set.endArray();

        return true;
    }

    return false;
}

/* --------------------------------------------------------------------- */

Settings::Settings()
{
    QSettings set;

    mSingleClickOpen = set.value("single_click", false).toBool();
    mChangeDir = set.value("change_dir", true).toBool();

#ifdef Q_OS_WIN
    mVimCommand = set.value("vim_command", "C:/Program Files/Vim/vim74/gvim.exe").toString();
#else
    mVimCommand = set.value("vim_command", "gvim").toString();
#endif  // Q_OS_WIN

    mOpenNewTabs = set.value("new_tabs", true).toBool();
    mDarkBackground = set.value("dark_background", false).toBool();
    mShowHidden = set.value("show_hidden", false).toBool();
    mAutoQuickList = set.value("auto_quicklist", false).toBool();
    mAsToolWindow = set.value("as_tool_window", false).toBool();
    mUseSessions = set.value("use_sessions", false).toBool();
    mOneVimWindow = set.value("one_vim_window", false).toBool();
    mProfilesIcons = set.value("profile_icon_view", false).toBool();

    mLanguage = set.value("language", QLocale::system().name()).toString();
    mLastProfile = set.value("last_profile", 0).toInt();

#ifdef Q_OS_WIN
    mSessionFileDir = set.value("session_file_dir", QDir::home().absoluteFilePath("_vimpal/sessions")).toString();
#else
    mSessionFileDir = set.value("session_file_dir", QDir::home().absoluteFilePath(".local/share/vimpal/sessions")).toString();
#endif  // Q_OS_WIN

    const int c = set.beginReadArray("tools");

    for (int i=0; i<c; i++)
    {
        set.setArrayIndex(i);

        Tool t;

        t.Name = set.value("name").toString();
        t.Command = set.value("command").toString();
        t.Filter = set.value("filter").toString();
        t.ByDefault = set.value("by_default", false).toBool();

        mTools.append(t);
    }

    set.endArray();
}
