/*  This file is part of VIM Pal.
 *
 *  VIM Pal is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  VIM Pal is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with VIM Pal.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "settingsdialog.h"
#include "tooldialog.h"

#include <QFileDialog>
#include <QPushButton>
#include <QStandardItemModel>


const QStringList SettingsDialog::sLanguageNames = QStringList()
               << "English" << "French" << "Lithuanian";

const QStringList SettingsDialog::sLanguageCodes = QStringList()
               << "en_EN" << "fr_FR" << "lt_LT";


SettingsDialog::SettingsDialog(QWidget *parent)
   :QDialog(parent)
{
    setupUi(this);

    mToolModel = new QStandardItemModel(this);

    toolView->setModel(mToolModel);

    buttonBox->button(QDialogButtonBox::Ok)->setText(tr("&OK"));
    buttonBox->button(QDialogButtonBox::Cancel)->setText(tr("&Cancel"));

    connect(toolView->selectionModel(), SIGNAL(currentChanged(QModelIndex, QModelIndex)),
            this, SLOT(currentToolChanged(QModelIndex, QModelIndex)));

    language->addItems(sLanguageNames);
}

void SettingsDialog::setTools(const QList<Settings::Tool> &tools)
{
    foreach (const Settings::Tool& t, tools)
    {
        QList<QStandardItem*> item;

        item.append(new QStandardItem(t.Name));
        item.append(new QStandardItem(t.Command));
        item.append(new QStandardItem(t.Filter));
        item.append(new QStandardItem(t.ByDefault ? "1" : "0"));

        mToolModel->appendRow(item);
    }
}

const QList<Settings::Tool> SettingsDialog::getTools() const
{
    QList<Settings::Tool> list;

    for (int i=0, c=mToolModel->rowCount(); i<c; i++)
    {
        Settings::Tool t;

        t.Name = mToolModel->index(i, 0).data(Qt::DisplayRole).toString();
        t.Command = mToolModel->index(i, 1).data(Qt::DisplayRole).toString();
        t.Filter = mToolModel->index(i, 2).data(Qt::DisplayRole).toString();
        t.ByDefault = mToolModel->index(i, 3).data(Qt::DisplayRole).toString().toInt();

        list.append(t);
    }

    return list;
}

void SettingsDialog::setLanguage(const QString &lang)
{
    const int idx = sLanguageCodes.indexOf(lang);

    if (idx != -1)
    {
        language->setCurrentIndex(idx);
    }
}

QString SettingsDialog::getLanguage() const
{
    const int idx = language->currentIndex();

    return sLanguageCodes.at(idx >= 0 && idx < sLanguageCodes.length() ? idx : 0);
}

/* --------------------------------------------------------------------- */

void SettingsDialog::on_browseCmd_clicked()
{
    const QString file = QFileDialog::getOpenFileName(this, tr("Find GVIM executable"), vimCommand->text());

    if (!file.isEmpty())
    {
        vimCommand->setText(file);
    }
}

void SettingsDialog::on_addTool_clicked()
{
    ToolDialog dlg(this);

    if (dlg.exec() == QDialog::Accepted)
    {
        QList<QStandardItem*> item;

        item.append(new QStandardItem(dlg.name->text()));
        item.append(new QStandardItem(dlg.command->text()));
        item.append(new QStandardItem(dlg.filter->text()));
        item.append(new QStandardItem(dlg.byDefault->isChecked() ? "1" : "0"));

        mToolModel->appendRow(item);
    }
}

void SettingsDialog::on_removeTool_clicked()
{
    if (toolView->currentIndex().isValid())
    {
        mToolModel->removeRow(toolView->currentIndex().row());

        currentToolChanged(toolView->currentIndex());
    }
}

void SettingsDialog::on_moveToolUp_clicked()
{
    if (toolView->currentIndex().isValid())
    {
        const int row = toolView->currentIndex().row();

        if (row > 0)
        {
            mToolModel->insertRow(row - 1, mToolModel->takeRow(row));

            toolView->selectionModel()->setCurrentIndex(mToolModel->index(row - 1, 0),
                QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Current | QItemSelectionModel::Rows);
        }
    }
}

void SettingsDialog::on_moveToolDown_clicked()
{
    if (toolView->currentIndex().isValid())
    {
        const int row = toolView->currentIndex().row();

        if (row < mToolModel->rowCount() - 1)
        {
            mToolModel->insertRow(row + 1, mToolModel->takeRow(row));

            toolView->selectionModel()->setCurrentIndex(mToolModel->index(row + 1, 0),
                QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Current | QItemSelectionModel::Rows);
        }
    }
}

void SettingsDialog::on_browseSessionDir_clicked()
{
    const QString dir = QFileDialog::getExistingDirectory(this, tr("Select directory for session files"),
                                                          sessionDir->text());

    if (!dir.isEmpty())
    {
        sessionDir->setText(dir);
    }
}

void SettingsDialog::currentToolChanged(const QModelIndex &index, const QModelIndex &previous)
{
    Q_UNUSED(previous);

    if (index.isValid())
    {
        removeTool->setEnabled(true);
        moveToolUp->setEnabled(index.row() > 0);
        moveToolDown->setEnabled(index.row() < mToolModel->rowCount() - 1);
    }
    else
    {
        removeTool->setEnabled(false);
        moveToolUp->setEnabled(false);
        moveToolDown->setEnabled(false);
    }
}

void SettingsDialog::on_toolView_activated(const QModelIndex &index)
{
    ToolDialog dlg(this);

    QModelIndex name = mToolModel->index(index.row(), 0),
                cmd = mToolModel->index(index.row(), 1),
                filt = mToolModel->index(index.row(), 2),
                def = mToolModel->index(index.row(), 3);

    dlg.name->setText(name.data().toString());
    dlg.command->setText(cmd.data().toString());
    dlg.filter->setText(filt.data().toString());
    dlg.byDefault->setChecked(def.data().toString().toInt());

    if (dlg.exec() == QDialog::Accepted)
    {
        mToolModel->setData(name, dlg.name->text(), Qt::DisplayRole);
        mToolModel->setData(cmd, dlg.command->text(), Qt::DisplayRole);
        mToolModel->setData(filt, dlg.filter->text(), Qt::DisplayRole);
        mToolModel->setData(def, dlg.byDefault->isChecked() ? "1" : "0", Qt::DisplayRole);
    }
}
