/*  This file is part of VIM Pal.
 *
 *  VIM Pal is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  VIM Pal is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with VIM Pal.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "vimpal.h"
#include "version.h"
#include "profilemodel.h"
#include "profile.h"
#include "settings.h"
#include "quickfileview.h"
#include "vimutils.h"
#include "fileutils.h"

#include <QCloseEvent>
#include <QComboBox>
#include <QFileSystemModel>
#include <QMenu>
#include <QMessageBox>
#include <QSettings>
#include <QTimer>
#include <QToolBar>
#include <QToolButton>
#include <QTreeView>
#include <QPushButton>
#include <QInputDialog>
#include <QDir>
#include <QDockWidget>
#include <QTreeWidget>
#include <QTranslator>
#include <QListWidget>

#include <QHeaderView>
#include <QApplication>
#include <QProcess>


VimPal::VimPal(QWidget* parent, Qt::WindowFlags f)
    :QMainWindow(parent, f), mCurrentProfile(0), mCurrentTranslator(0)
{
    setObjectName("VimPal");
    setWindowIcon(QIcon(":/img/icon.png"));

    mProfileModel = new ProfileModel(this);

    // Common actions
    mRemoveAction = new QAction(this);
    mEditAction = new QAction(this);

    mRemoveAction->setShortcut(QKeySequence("Del"));
    mEditAction->setShortcut(QKeySequence("F2"));

    connect(mRemoveAction, SIGNAL(triggered()), this, SLOT(removeItems()));
    connect(mEditAction, SIGNAL(triggered()), this, SLOT(editItems()));

    addAction(mEditAction);
    addAction(mRemoveAction);

    // Navigation toolbal

    mNavigationToolbar = new QToolBar(this);
    mStartBtn = new QToolButton(this);
    mPreferencesAction = new QAction(this);
    mAboutAction = new QAction(this);
    mQuitAction = new QAction(this);
    mGoBack = new QAction(this);
    mGoForward = new QAction(this);
    mGoHome = new QAction(this);
    mGoUpBtn = new QToolButton(this);
    mRootMenu = new QMenu(this);

    mNavigationToolbar->setIconSize(QSize(22, 22));
    mNavigationToolbar->setObjectName("navigationToolbar");
    mQuitAction->setShortcut(QKeySequence("Ctrl+Q"));
    mGoBack->setIcon(QIcon::fromTheme("go-previous", QIcon(":/img/go-previous.png")));
    mGoBack->setShortcut(QKeySequence("Alt+Left"));
    mGoForward->setIcon(QIcon::fromTheme("go-next", QIcon(":/img/go-next.png")));
    mGoForward->setShortcut(QKeySequence("Alt+Right"));
    mGoHome->setIcon(QIcon::fromTheme("go-home", QIcon(":/img/go-home.png")));
    mGoHome->setShortcut(QKeySequence("Alt+Home"));
    mStartBtn->setIcon(QIcon(":/img/icon.png"));
    mStartBtn->setPopupMode(QToolButton::InstantPopup);
    mGoUpBtn->setIcon(QIcon::fromTheme("go-up", QIcon(":/img/go-up.png")));
    mGoUpBtn->setShortcut(QKeySequence("Alt+Up"));
    mGoUpBtn->setPopupMode(QToolButton::MenuButtonPopup);
    mGoUpBtn->setMenu(mRootMenu);

    connect(mPreferencesAction, SIGNAL(triggered()), this, SLOT(editPreferences()));
    connect(mAboutAction, SIGNAL(triggered()), this, SLOT(showAbout()));
    connect(mQuitAction, SIGNAL(triggered()), this, SLOT(close()));
    connect(mGoBack, SIGNAL(triggered()), this, SLOT(goBack()));
    connect(mGoForward, SIGNAL(triggered()), this, SLOT(goForward()));
    connect(mGoHome, SIGNAL(triggered()), this, SLOT(goHome()));
    connect(mRootMenu, SIGNAL(aboutToShow()), this, SLOT(updateRootMenu()));
    connect(mRootMenu, SIGNAL(triggered(QAction*)), this, SLOT(changeRoot(QAction*)));
    connect(mGoUpBtn, SIGNAL(clicked()), this, SLOT(goUp()));

    {
        QMenu* m = new QMenu(this);

        m->addAction(mPreferencesAction);
        m->addSeparator();
        m->addAction(mAboutAction);
        m->addSeparator();
        m->addAction(mQuitAction);

        mStartBtn->setMenu(m);
    }

    mNavigationToolbar->addWidget(mStartBtn);
    mNavigationToolbar->addSeparator();
    mNavigationToolbar->addAction(mGoBack);
    mNavigationToolbar->addAction(mGoForward);
    mNavigationToolbar->addWidget(mGoUpBtn);
    mNavigationToolbar->addAction(mGoHome);

    addToolBar(Qt::TopToolBarArea, mNavigationToolbar);

    // Profile toolbar

    mProfileToolbar = new QToolBar(this);
    mProfileBox = new QComboBox(this);

    mProfileToolbar->setObjectName("profileToolbar");
    mProfileBox->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
    mProfileBox->setModel(mProfileModel);

    connect(mProfileBox, SIGNAL(currentIndexChanged(int)), this, SLOT(setProfile(int)));

    {
        QToolButton* tb = new QToolButton(this);

        mProfileToolbar->addWidget(mProfileBox);
        mProfileToolbar->addWidget(tb);

        tb->setText("...");

        connect(tb, SIGNAL(clicked()), this, SLOT(editProfiles()));
    }

    addToolBar(Qt::BottomToolBarArea, mProfileToolbar);

    // Profile dock

    mProfileDock = new QDockWidget(this);
    mProfileView = new QListView(this);

    mProfileDock->setObjectName("profileDock");
    mProfileDock->hide();
    mProfileView->setContextMenuPolicy(Qt::ActionsContextMenu);
    mProfileView->setModel(mProfileModel);

    connect(mProfileView, SIGNAL(clicked(QModelIndex)), this, SLOT(profileClicked(QModelIndex)));
    connect(mProfileView, SIGNAL(activated(QModelIndex)), this, SLOT(profileClicked(QModelIndex)));

    mProfileDock->setWidget(mProfileView);

    addDockWidget(Qt::TopDockWidgetArea, mProfileDock);

    mProfileView->setViewMode(Settings::instance()->profilesIcons() ? QListView::IconMode : QListView::ListMode);
    mProfileView->setUniformItemSizes(true);

    // Quick list dock

    mQuickListDock = new QDockWidget(this);
    mQuickFileView = new QuickFileView(this);

    mQuickListDock->setObjectName("quickFileDock");
    mQuickListDock->setWidget(mQuickFileView);
    mQuickListDock->hide();

    connect(this, SIGNAL(profileChanged(Profile*)), mQuickFileView, SLOT(setProfile(Profile*)));

    addDockWidget(Qt::TopDockWidgetArea, mQuickListDock);

    // Tree view

    mDirView = new QTreeView(this);
    mDirModel = new QFileSystemModel(this);

    connect(mDirView, SIGNAL(clicked(QModelIndex)), this, SLOT(fileClick(QModelIndex)));
    connect(mDirView, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(fileDblClick(QModelIndex)));
    connect(mDirView, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(fileTreeContextMenu(QPoint)));

    setCentralWidget(mDirView);

    mDirView->setModel(mDirModel);
    mDirView->hideColumn(1);
    mDirView->hideColumn(2);
    mDirView->hideColumn(3);
#if QT_VERSION >= 0x050000
    mDirView->header()->setSectionResizeMode(QHeaderView::ResizeToContents);
#else
    mDirView->header()->setResizeMode(QHeaderView::ResizeToContents);
#endif
    mDirView->setHeaderHidden(true);
    mDirView->setContextMenuPolicy(Qt::CustomContextMenu);

    mDirModel->sort(0);

    FileUtils::init(this);
    VimUtils::init(this);

    // Load settings

    setLanguage(Settings::instance()->language());
    setProfile(Settings::instance()->lastProfile());

    setWindowFlags(Settings::instance()->asToolWindow() ? Qt::Tool : Qt::Window);

    updateColors();
    clearHistory();

    mDirModel->setFilter(Settings::instance()->showHidden() ?
                             (QDir::AllEntries | QDir::AllDirs | QDir::NoDotAndDotDot | QDir::Hidden) :
                             (QDir::AllEntries | QDir::AllDirs | QDir::NoDotAndDotDot));

    {
        QSettings set;

        restoreGeometry(set.value("window/geometry").toByteArray());
        restoreState(set.value("window/state").toByteArray());

        // Legacy settings (dropped in v1.5.0).
        if (set.contains("window/expanded"))
        {
            foreach (const QString& dir, set.value("window/expanded", QStringList()).toStringList())
            {
                const QModelIndex idx = mDirModel->index(dir);

                if (idx.isValid())
                {
                    mDirView->expand(idx);
                }
            }

            set.remove("window/expanded");
        }
    }
}

QSize VimPal::sizeHint() const
{
    return QSize(260, 640);
}

/* --------------------------------------------------------------------- */

void VimPal::closeEvent(QCloseEvent* e)
{
    QSettings set;

    set.setValue("window/geometry", saveGeometry());
    set.setValue("window/state", saveState());

    if (mCurrentProfile)
    {
        mCurrentProfile->setExpandedDirs(getExpanded(mDirView->rootIndex()));
        mQuickFileView->saveExpandedDirs();

        if (Settings::instance()->useSessions())
        {
            VimUtils::writeSession();

            if (Settings::instance()->oneVimWindow())
            {
                VimUtils::close();
            }
        }
    }

    e->accept();

    QApplication::instance()->quit();
}

void VimPal::changeEvent(QEvent *e)
{
    if (e->type() == QEvent::LanguageChange)
    {
        retranslateUi();
    }
}

/* --------------------------------------------------------------------- */

void VimPal::goHome()
{
    selectDir(QDir::homePath());
}

void VimPal::goUp()
{
    if (mDirView->rootIndex().parent().isValid())
    {
        selectDir(mDirView->rootIndex().parent());
    }
}

void VimPal::goBack()
{
    if (!mHistory.isEmpty())
    {
        const QString p = mHistory.pop();
        mForwardHistory.push(mDirModel->rootPath());

        mGoForward->setEnabled(true);

        if (mHistory.isEmpty())
        {
            mGoBack->setDisabled(true);
        }

        selectDir(p, true);
    }
}

void VimPal::goForward()
{
    if (!mForwardHistory.isEmpty())
    {
        const QString p = mForwardHistory.pop();
        mHistory.push(mDirModel->rootPath());

        mGoBack->setEnabled(true);

        if (mForwardHistory.isEmpty())
        {
            mGoForward->setDisabled(true);
        }

        selectDir(p, true);
    }
}

void VimPal::changeRoot(QAction *action)
{
    if (!action)
    {
        return;
    }

    const QString root = action->property("path").toString();

    if (!root.isEmpty())
    {
        selectDir(root);
    }
}

void VimPal::updateRootMenu()
{
    mRootMenu->clear();

    foreach (const QFileInfo& drive, QDir::drives())
    {
        mRootMenu->addAction(drive.absoluteFilePath())->setProperty("path", drive.absoluteFilePath());
    }
}

void VimPal::fileClick(const QModelIndex &idx)
{
    if (!Settings::instance()->singleClickOpen())
    {
        return;
    }

    if (mDirModel->isDir(idx))
    {
        if (Settings::instance()->changeDir())
        {
            VimUtils::cd(mDirModel->filePath(idx));
        }
    }
    else
    {
        selectFile(idx);
    }
}

void VimPal::fileDblClick(const QModelIndex &idx)
{
    if (mDirModel->isDir(idx))
    {
        selectDir(idx);
    }
    else
    {
        selectFile(idx);
    }
}

void VimPal::fileTreeContextMenu(const QPoint &pos)
{
    QModelIndex idx = mDirView->indexAt(pos);

    QMenu m(this),
            newMenu(tr("&New"), this),
            toolMenu(tr("Toolbars"), this);

    QMap<QAction*,int> actions;
    QMap<QAction*,QString> commands;

    toolMenu.addAction(mNavigationToolbar->toggleViewAction());
    toolMenu.addAction(mProfileToolbar->toggleViewAction());
    toolMenu.addSeparator();
    toolMenu.addAction(mProfileDock->toggleViewAction());
    toolMenu.addAction(mQuickListDock->toggleViewAction());

    actions.insert(newMenu.addAction(tr("&File...")), 6);
    actions.insert(newMenu.addAction(tr("&Directory...")), 7);

    if (idx.isValid())
    {
        mDirView->selectionModel()->select(idx, QItemSelectionModel::ClearAndSelect);

        if (mDirModel->isDir(idx))
        {
            actions.insert(m.addAction(tr("&Set as root")), 2);
            actions.insert(m.addAction(tr("&Set as current in Vim")), 1);

            m.addSeparator();

            m.addMenu(&newMenu);
        }
        else
        {
            FileUtils::instance()->addFileActions(mDirModel->fileInfo(idx).absoluteFilePath(), &m);

            actions.insert(m.addAction(tr("Add to &quicklist")), 8);

            m.addSeparator();
        }

        actions.insert(m.addAction(tr("&Rename...")), 4);
        actions.insert(m.addAction(tr("&Delete...")), 5);
    }
    else
    {
        mDirView->selectionModel()->clear();

        actions.insert(m.addAction(tr("&Set as current in Vim")), 1);

        m.addSeparator();
        m.addMenu(&newMenu);

        idx = mDirView->rootIndex();
    }

    m.addSeparator();
    m.addMenu(&toolMenu);
    m.addAction(tr("&Preferences..."), this, SLOT(editPreferences()));
    m.addAction(tr("Pro&files..."), this, SLOT(editProfiles()));

    QAction* a = m.exec(mDirView->mapToGlobal(pos) + QPoint(1, 1));

    switch (actions.value(a, -1))
    {
        case -1:
        {
            const QString cmd = commands.value(a);

            if (!cmd.isEmpty())
            {
                if (!QProcess::startDetached(cmd))
                {
                    QMessageBox::critical(this, tr("Error"), tr("Failed to launch external tool."));
                }
            }

            break;
        }

        case 1:
            VimUtils::cd(mDirModel->filePath(idx));
            break;

        case 2:
            selectDir(idx);
            break;

        case 3:
            selectFile(idx);
            break;

        case 4:
            renameFile(idx);
            break;

        case 5:
            deleteFile(idx);
            break;

        case 6:
            newFile(idx);
            break;

        case 7:
            newDirectory(idx);
            break;

        case 8:
            mQuickFileView->addFile(mDirModel->filePath(idx));
            break;
    }
}

void VimPal::editPreferences()
{
    const bool prevAsTool = Settings::instance()->asToolWindow();
    const QString prevLang = Settings::instance()->language();

    QVariantMap extra;

    extra["navBar"] = mNavigationToolbar->isVisible();
    extra["profileBar"] = mProfileToolbar->isVisible();
    extra["profileDock"] = mProfileDock->isVisible();
    extra["quickDock"] = mQuickListDock->isVisible();

    if (Settings::instance()->settingsDialog(this, extra))
    {
        if (prevLang != Settings::instance()->language())
        {
            setLanguage(Settings::instance()->language());
        }

        if (prevAsTool != Settings::instance()->asToolWindow())
        {
            setWindowFlags(Settings::instance()->asToolWindow() ? Qt::Tool : Qt::Window);
            show();
        }

        mNavigationToolbar->setVisible(extra["navBar"].toBool());
        mProfileToolbar->setVisible(extra["profileBar"].toBool());
        mProfileDock->setVisible(extra["profileDock"].toBool());
        mQuickListDock->setVisible(extra["quickDock"].toBool());

        updateColors();

        mDirModel->setFilter(Settings::instance()->showHidden() ?
                                 (QDir::AllEntries | QDir::AllDirs | QDir::NoDotAndDotDot | QDir::Hidden) :
                                 (QDir::AllEntries | QDir::AllDirs | QDir::NoDotAndDotDot));

        mProfileView->setViewMode(Settings::instance()->profilesIcons() ? QListView::IconMode : QListView::ListMode);
    }
}

void VimPal::editProfiles()
{
    int index = mCurrentProfile->data().Index;

    mProfileBox->blockSignals(true);
    mProfileView->blockSignals(true);

    if (mProfileModel->profilesDialog(this, index))
    {
        if (!mProfileModel->profileExists(mCurrentProfile))
        {
            // Current profile has been deleted.

            mCurrentProfile = 0;
            mQuickFileView->resetProfile();
        }

        setProfile(index);

        // In case profile name changed...
        VimUtils::setServer(mCurrentProfile->data().Server);
    }

    mProfileBox->blockSignals(false);
    mProfileView->blockSignals(false);
}

void VimPal::editItems()
{
    if (mProfileView->hasFocus())
    {
        editProfiles();
    }
    else if (mDirView->hasFocus())
    {
        QModelIndex idx = mDirView->currentIndex();

        if (idx.isValid())
        {
            renameFile(idx);
        }
    }
    else if (mQuickFileView->hasFocus())
    {
        mQuickFileView->editCurrent();
    }
}

void VimPal::removeItems()
{
    if (mDirView->hasFocus())
    {
        QModelIndex idx = mDirView->currentIndex();

        if (idx.isValid())
        {
            deleteFile(idx);
        }
    }
    else if (mQuickFileView->hasFocus())
    {
        mQuickFileView->removeCurrent();
    }
}

void VimPal::setProfile(int prof)
{
    // Get pointer
    Profile* p = mProfileModel->profile(prof);

    if (!p)
    {
        qWarning("Trying to set a non-existing profile. This means there's a bug somewhere.");

        return;
    }

    // Update UI selectors
    if (mProfileBox->currentIndex() != prof)
    {
        mProfileBox->blockSignals(true);
        mProfileBox->setCurrentIndex(prof);
        mProfileBox->blockSignals(false);
    }

    if (mProfileView->currentIndex().row() != prof)
    {
        mProfileView->blockSignals(true);
        mProfileView->selectionModel()->setCurrentIndex(mProfileView->model()->index(prof, 0), QItemSelectionModel::ClearAndSelect);
        mProfileView->blockSignals(false);
    }

    // Current profile changed.
    if (mCurrentProfile != p)
    {
        if (mCurrentProfile)
        {
            // Save old profile state
            mCurrentProfile->setExpandedDirs(getExpanded(mDirView->rootIndex()));

            if (Settings::instance()->useSessions())
            {
                VimUtils::writeSession();

                if (Settings::instance()->oneVimWindow())
                {
                    VimUtils::close();
                }
            }
        }

        mCurrentProfile = p;

        // Set file view root
        if (!p->data().Dir.isEmpty())
        {
            mDirModel->setRootPath(p->data().Dir);
            mDirView->setRootIndex(mDirModel->index(p->data().Dir));
        }
        else
        {
            p->setDir(mDirModel->fileInfo(mDirView->rootIndex()).absoluteFilePath());
        }

        // Restore expanded directories
        mDirView->collapseAll();

        foreach (const QString& dir, mCurrentProfile->data().ExpandedDirs)
        {
            const QModelIndex idx = mDirModel->index(dir);

            if (idx.isValid())
            {
                mDirView->expand(idx);
            }
        }

        // Save new current profile index
        QSettings set;

        set.setValue("last_profile", prof);

        clearHistory();

        emit profileChanged(p);

        VimUtils::setServer(p->data().Server);

        if (Settings::instance()->useSessions() && Settings::instance()->oneVimWindow())
        {
            VimUtils::startSession();
        }
    }

    VimUtils::activate();
}

void VimPal::profileClicked(const QModelIndex &idx)
{
    setProfile(idx.row());
}

void VimPal::showAbout()
{
    QMessageBox::about(this, tr("VIM Pal"),
                       tr("<big><strong>VIM Pal</strong></big><br /><em>Version %1</em><br /><br />&copy; %2").arg(VERSION).arg(COPYRIGHT));
}

/* --------------------------------------------------------------------- */

void VimPal::selectFile(const QModelIndex &idx)
{
    const QString fname = mDirModel->filePath(idx);

    FileUtils::instance()->openFile(fname);

    if (Settings::instance()->autoQuickList())
    {
        mQuickFileView->addFile(fname);
    }
}

void VimPal::selectDir(const QModelIndex &idx, bool noHistory)
{
    if (idx != mDirView->rootIndex())
    {
        selectDir(idx, mDirModel->fileInfo(idx).absoluteFilePath(), noHistory);
    }
}

void VimPal::selectDir(const QString &dir, bool noHistory)
{
    if (dir != mDirModel->rootPath())
    {
        selectDir(mDirModel->index(dir), dir, noHistory);
    }
}

void VimPal::selectDir(const QModelIndex &idx, const QString &dir, bool noHistory)
{
    if (!noHistory)
    {
        pushHistory(mDirModel->rootPath());
    }

    mDirModel->setRootPath(dir);
    mDirView->setRootIndex(idx);

    mCurrentProfile->setDir(dir);
}

void VimPal::deleteFile(const QModelIndex &idx)
{
    const QFileInfo info = mDirModel->fileInfo(idx);
    QString msg;
    QStringList files, dirs;

    if (info.isDir())
    {
        QDirIterator it(info.absoluteFilePath(), QDirIterator::Subdirectories);

        while (!it.next().isEmpty())
        {
            if (it.fileName() == "." || it.fileName() == "..")
            {
                continue;
            }

            const QFileInfo in = it.fileInfo();

            if (in.isDir())
            {
                dirs.prepend(in.absoluteFilePath());
            }
            else
            {
                files.append(in.absoluteFilePath());
            }
        }

        if (files.isEmpty() && dirs.isEmpty())
        {
            msg = tr("Are you sure you want to delete directory <b>%1</b> (empty)?").arg(info.fileName());
        }
        else
        {
            msg = tr("Are you sure you want to delete non-empty directory <b>%1</b> (files: %2, directories: %3)?").arg(info.fileName()).arg(files.count()).arg(dirs.count());
        }

        dirs.append(info.absoluteFilePath());
    }
    else
    {
        files.append(info.absoluteFilePath());

        msg = tr("Are you sure you want to delete file <b>%1</b> (size: %2B)?").arg(info.fileName()).arg(info.size());
    }

    QMessageBox box(QMessageBox::Warning, tr("Confirm delete"), msg, QMessageBox::Yes | QMessageBox::No, this);

    box.button(QMessageBox::Yes)->setText(tr("&Yes"));
    box.button(QMessageBox::No)->setText(tr("&No"));

    if (box.exec() == QMessageBox::Yes)
    {
        foreach (const QString& file, files)
        {
            QFile f(file);

            if (!f.remove())
            {
                QMessageBox::critical(this, tr("Error"), tr("Unable to remove file: %1").arg(f.errorString()));

                return;
            }

            // TODO: remove from quick-files?
        }

        foreach (const QString& dir, dirs)
        {
            if (!mDirModel->rmdir(mDirModel->index(dir)))
            {
                QMessageBox::critical(this, tr("Error"), tr("Unable to remove directory."));

                return;
            }
        }
    }
}

void VimPal::renameFile(const QModelIndex &idx)
{
    const QFileInfo info = mDirModel->fileInfo(idx);

    QInputDialog dlg(this);

    dlg.setOkButtonText(tr("&OK"));
    dlg.setCancelButtonText(tr("&Cancel"));
    dlg.setWindowTitle(info.isDir() ? tr("Rename directory") : tr("Rename file"));
    dlg.setLabelText(info.isDir() ? tr("Enter new directory name:") : tr("Enter new file name:"));
    dlg.setTextValue(info.fileName());

    if (dlg.exec() == QDialog::Accepted)
    {
        QFile f(info.absoluteFilePath());

        if (!f.rename(info.absoluteDir().absoluteFilePath(dlg.textValue())))
        {
            QMessageBox::critical(this, tr("Error"), tr("Unable to rename: %1").arg(f.errorString()));
        }
    }
}

void VimPal::newFile(const QModelIndex &idx)
{
    QInputDialog dlg(this);

    dlg.setOkButtonText(tr("&OK"));
    dlg.setCancelButtonText(tr("&Cancel"));
    dlg.setWindowTitle(tr("Create new file"));
    dlg.setLabelText(tr("Enter the name for new file:"));

    if (dlg.exec() == QDialog::Accepted)
    {
        const QString name = getDir(idx) + "/" + dlg.textValue();

        if (QFile::exists(name))
        {
            QMessageBox::critical(this, tr("Error"), tr("File already exists."));
        }
        else
        {
            QFile f(name);

            if (f.open(QIODevice::WriteOnly))
            {
                f.close();
            }
            else
            {
                QMessageBox::critical(this, tr("Error"), tr("Unable to create file: %1").arg(f.errorString()));
            }
        }
    }
}

void VimPal::newDirectory(const QModelIndex &idx)
{
    QInputDialog dlg(this);

    dlg.setOkButtonText(tr("&OK"));
    dlg.setCancelButtonText(tr("&Cancel"));
    dlg.setWindowTitle(tr("Create new directory"));
    dlg.setLabelText(tr("Enter the name for new directory:"));

    if (dlg.exec() == QDialog::Accepted)
    {
        if (!mDirModel->mkdir(mDirModel->index(getDir(idx)), dlg.textValue()).isValid())
        {
            QMessageBox::critical(this, tr("Error"), tr("Unable to create directory."));
        }
    }
}

void VimPal::updateColors()
{
    if (Settings::instance()->darkBackground())
    {
        mDirView->setStyleSheet("background-color: #222; color: #EEE;");
    }
    else
    {
        mDirView->setStyleSheet("");
    }
}

void VimPal::clearHistory()
{
    mGoBack->setDisabled(true);
    mGoForward->setDisabled(true);
    mHistory.clear();
    mForwardHistory.clear();
}

void VimPal::pushHistory(const QString &path)
{
    if (mHistory.isEmpty() || mHistory.top() != path)
    {
        mGoBack->setEnabled(true);
        mHistory.push(path);

        mGoForward->setDisabled(true);
        mForwardHistory.clear();
    }
}

void VimPal::setLanguage(const QString &lang)
{
    if (mCurrentTranslator)
    {
        QApplication::instance()->removeTranslator(mCurrentTranslator);

        mCurrentTranslator->deleteLater();
        mCurrentTranslator = 0;
    }

    if (QFile::exists(QString(":/i18n/%1.qm").arg(lang)))
    {
        mCurrentTranslator = new QTranslator(QApplication::instance());

        if (mCurrentTranslator->load(QString(":/i18n/%1.qm").arg(lang)))
        {
            QApplication::instance()->installTranslator(mCurrentTranslator);
        }
        else
        {
            mCurrentTranslator->deleteLater();
            mCurrentTranslator = 0;

            qWarning("Unable to load translation.");
        }
    }

    retranslateUi();
}

QString VimPal::getDir(const QModelIndex &idx)
{
    const QFileInfo info = mDirModel->fileInfo(idx.isValid() ? idx : mDirView->rootIndex());

    return info.isDir() ? info.absoluteFilePath() : info.absolutePath();
}

QStringList VimPal::getExpanded(const QModelIndex &parent)
{
    if (parent.isValid())
    {
        QStringList list;

        const int c = parent.model()->rowCount(parent);

        for (int i=0; i<c; i++)
        {
            const QModelIndex idx = parent.child(i, 0);

            if (mDirView->isExpanded(idx))
            {
                list.append(mDirModel->fileInfo(idx).absoluteFilePath());
                list.append(getExpanded(idx));
            }
        }

        return list;
    }
    else
    {
        return QStringList();
    }
}

void VimPal::retranslateUi()
{
    setWindowTitle(tr("VIM Pal"));

    mPreferencesAction->setText(tr("&Preferences..."));
    mAboutAction->setText(tr("&About VIM Pal..."));
    mQuitAction->setText(tr("&Quit"));
    mGoBack->setText(tr("Go back"));
    mGoForward->setText(tr("Go forward"));
    mGoHome->setText(tr("Go home"));

    mGoUpBtn->setToolTip(tr("Go up"));

    mNavigationToolbar->setWindowTitle(tr("Navigation"));
    mProfileToolbar->setWindowTitle(tr("Profile"));

    mProfileDock->setWindowTitle(tr("Profile selector"));
    mQuickListDock->setWindowTitle(tr("Quick file selector"));
}
