/*  This file is part of VIM Pal.
 *
 *  VIM Pal is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  VIM Pal is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with VIM Pal.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "vimutils.h"
#include "settings.h"

#include <QProcess>
#include <QFileInfo>
#include <QDir>
#include <QEventLoop>
#include <QTimer>
#include <QMessageBox>


QString VimUtils::sServer;
QWidget* VimUtils::sParent = 0;


void VimUtils::init(QWidget *parent)
{
    sParent = parent;
}

void VimUtils::setServer(const QString &sname)
{
    sServer = sname;
}

bool VimUtils::openFile(const QString &fname)
{
    if (Settings::instance()->useSessions())
    {
        if (!startSession())
        {
            return false;
        }
    }

    if (Settings::instance()->openNewTabs())
    {
        if (!cmd(QString("--remote-tab \"%1\"").arg(fname), true))
        {
            return false;
        }
    }
    else
    {
        if (!cmd(QString("--remote \"%1\"").arg(fname), true))
        {
            return false;
        }
    }

    if (Settings::instance()->changeDir())
    {
        cd(QFileInfo(fname).dir().absolutePath());
    }

    return true;
}

void VimUtils::activate()
{
    cmd("--remote-send \"<C-\\><C-N>:silent! call foreground()<CR>\"", false);
}

void VimUtils::writeSession()
{
    if (serverExists())
    {
        QDir dir(Settings::instance()->sessionFileDir());

        if (!dir.exists())
        {
            if (!QDir::root().mkpath(dir.absolutePath()))
            {
                qWarning("Unable to create directory for session files.");

                return;
            }
        }

        cmd(QString("--remote-send \"<C-\\><C-N>:silent! mksession! %1<CR>\"").arg(dir.absoluteFilePath(sServer)), false);
    }
}

bool VimUtils::startSession()
{
    if (!serverExists())
    {
        const QString fname = QDir(Settings::instance()->sessionFileDir()).absoluteFilePath(sServer);

        if (QFile(fname).exists())
        {
            return cmd(QString("--remote-send \"<C-\\><C-N>:silent! source %1<CR><CR>\"").arg(fname), true);
        }
        else
        {
            return cmd("--remote-send \"<C-\\><C-N>:silent! call foreground()<CR>\"", true);
        }
    }
    else
    {
        return true;
    }
}

void VimUtils::close()
{
    if (serverExists())
    {
        cmd("--remote-send \"<C-\\><C-N>:silent! qa<CR>\"", false);
    }
}

void VimUtils::cd(const QString &dir)
{
    cmd(QString("--remote-send \"<C-\\><C-N>:silent! cd %1<CR>\"").arg(dir), false);
}

/* --------------------------------------------------------------------- */

bool VimUtils::cmd(const QString &cmd, bool startNew)
{
    bool ok = true, haveServer = serverExists();

    if (!haveServer && startNew)
    {
        QProcess* p = new QProcess;

        QObject::connect(p, SIGNAL(finished(int)), p, SLOT(deleteLater()));

        p->start(QString("\"%1\" --servername \"%2\"").arg(Settings::instance()->vimCommand()).arg(sServer));

        if (p->waitForStarted())
        {
            int checks = 6;

            while (--checks)
            {
                QEventLoop loop;
                QTimer::singleShot(500, &loop, SLOT(quit()));
                loop.exec();

                if (serverExists())
                {
                    haveServer = true;

                    break;
                }
            }
        }
        else
        {
            ok = false;
        }
    }

    if (haveServer)
    {
        QProcess* p = new QProcess;

        QObject::connect(p, SIGNAL(finished(int)), p, SLOT(deleteLater()));

        p->start(QString("\"%1\" --servername \"%2\" %3").arg(Settings::instance()->vimCommand()).arg(sServer).arg(cmd));

        if (!p->waitForStarted())
        {
            ok = false;
        }
    }
    else if (startNew)
    {
        ok = false;
    }

    if (!ok)
    {
        QMessageBox::critical(sParent, QObject::tr("Error"),
                              QObject::tr("Cannot execute VIM command. Please check your preferences and set the correct path to VIM executable."));
    }

    return ok;
}

bool VimUtils::serverExists()
{
    QProcess* p = new QProcess;
    bool ok = false;

#ifdef Q_OS_WIN
    // XXX: funny stuff on windows... as always
    {
        QString vcmd = Settings::instance()->vimCommand();

        vcmd.replace("gvim.exe", "vim.exe", Qt::CaseInsensitive);

        p->start(QString("\"%1\" --serverlist").arg(vcmd));
    }
#else
    p->start(QString("\"%1\" --serverlist").arg(Settings::instance()->vimCommand()));
#endif  // Q_WS_WIN

    if (p->waitForFinished())
    {
        while (!p->atEnd())
        {
            if (p->readLine().trimmed() == sServer)
            {
                ok = true;

                break;
            }
        }
    }

    p->deleteLater();

    return ok;
}
